################################################################################
# This is for determining whether there is a person effect in the measurements.
#
# EDIT FILE Include_setAnalysisAndDate.R BEFORE RUNNING THIS CODE.
################################################################################

################################################################################
# Set working directory.
################################################################################

setwd("/Users/tedtoal/Documents/UCDavis/BradyLab/GrowthExperiments/Tomato/RootRNAseqJun2013_Phenotypes")

################################################################################
# Include other files used by this.
################################################################################

source("Include_setAnalysisAndDate.R")
source("Include_general.R")
source("Include_xlsx.R")
source("Include_plateLayout.R")
source("Include_dataColumnTypes.R")

################################################################################
# Load libraries used by this.
################################################################################

library("lme4")

################################################################################
# Set basic constants controlling the analysis.
################################################################################

# Input directory for person effect data.
dataDir2 = "PersonEffectPenn"

# Output directory name.
outputFolder = "PersonEffectPenn"

# Input data filename.  The entire input data file is copied to the output folder.
inputFilename = paste(dataFilePrefix, processedDataFile, sep="")

# Input file pathnames.
processedDataFile1 = paste(dataDir, inputFilename, sep=PATHSEP)
processedDataFile2 = paste(dataDir2, inputFilename, sep=PATHSEP)

# Use TRUE to generate images tailored for a paper, FALSE for ourselves.
paper = FALSE

# Use FALSE to suppress annotation of number of samples on bar graphs.
addNumSampleAnnotation = !paper

# Use TRUE to dim the names of non-sig. ILs on X-axis label of bar graphs.
dimInsignificantNames = TRUE

# Use TRUE to color parent labels with parent colors.
colorParentLabels = TRUE

# Use FALSE to suppress error bars.
addErrorBars = TRUE

# Use FALSE to suppress "+" signs above bars indicating significance of uncorrected p-values.
showPlusForUncorrectP = !paper

# Use an empty list to not make any subset heatmaps, else each list element is a
# character vector of genotypes to include in the subset heatmap.
makeHeatmapSubsetGTs = list() # list(A=c("PENN", "IL2-3"), B=c("PENN", "IL7-5"))

# P-value breakpoints for increasing numbers of stars.
PvalueBreaks = c(0.001, 0.01, 0.05, 99)
theStars = c("***", "**", "*", "")
getStars = function(p) { p[is.na(p)] = 1; return(theStars[sapply(p, function(pi) match(TRUE, pi <= PvalueBreaks))]) }
thePlusses = c("+++", "++", "+", "")
getPlusses = function(p) { p[is.na(p)] = 1; return(thePlusses[sapply(p, function(pi) match(TRUE, pi <= PvalueBreaks))]) }
sigBarColors = c(`***`="gray40", `**`="gray55", `*`="gray70") # Corresponding to all but last PvalueBreaks entry.

# Bar graph bar colors.
colorTheseGenotypes = c() # Color these genotypes special colors.  Example: c("IL7-4-1", "IL7-5")
colorOfThoseGenotypes = c() # The colors to use.  Example: c("blue", "gold")
# Always include the parents in the special bar colors.
colorTheseGenotypes = c(colorTheseGenotypes, parents)
colorOfThoseGenotypes = c(colorOfThoseGenotypes, parentColorsLight)

# Set criteria for data filtering.
minPtsPerGenotypePerExperiment = 3

# Trait names, units, and descriptions.  The trait names are equal to the names
# of columns in the input data file containing the trait values.
mtx = matrix(ncol=3, byrow=TRUE, dimnames=list(NULL, c("trait", "units", "desc")), data=c(
#   trait           units       description
    "straightLen",  "cm",       "Root straight-line length from top to tip",
    "rootLen",      "cm",       "Root length from top to tip",
    "waviness",     "",         "Measure of waviness, 0 is a straight line",
    "wholeAng",     "degrees",  "Angle of straight line from root top to tip",
    "quarterAng",   "degrees",  "Angle of straight line from last quarter to tip",
    "tenthAng",     "degrees",  "Angle of straight line from last tenth to tip",
    "tipAng",       "degrees",  "Angle of tip of root",
    "avgAbsAng",    "degrees",  "Average of absolute value of angle along entire root",
    "wholeAngSign", "POS=rgt,NEG=lft",  "Left/right side of straight line from root top to tip",
    "quarterAngSign","POS=rgt,NEG=lft", "Left/right side of straight line from last quarter to tip",
    "tenthAngSign", "POS=rgt,NEG=lft",  "Left/right side of straight line from last tenth to tip",
    "tipAngSign",   "POS=rgt,NEG=lft",  "Left/right side of tip of root"))
dfTraits = as.data.frame(mtx[,2:ncol(mtx)], row.names=mtx[,1], stringsAsFactors=FALSE)
rm(mtx)
traits = rownames(dfTraits)

# Remove the "Sign" traits, we won't deal with those.
traits = traits[!grepl("Sign", traits)]
ntraits = length(traits)

################################################################################
# Create output file prefix and log file.
################################################################################

# Set output filename prefix, which prefixes all output files, and includes the
# output folder at the beginning of it.
rootOutputPrefix = paste(outputFolder, "", sep=PATHSEP)

# Create the output folder if it doesn't exist.
if (!file.exists(outputFolder))
    dir.create(outputFolder)

# Make an empty log file to start logging our activity into.
logfile = paste(rootOutputPrefix, "logfile.txt", sep="")
setDefaultLogFile(logfile)
printLog("RootRNAseqJune2013 phenotype modelling analysis log file", logfile, "\n", append=FALSE)
printLog("There are", ntraits, "traits in total:\n", getPrint(dfTraits, "    "), "\n")

# Save the dfTraits data frame.
filename = paste(rootOutputPrefix, "traits.tsv", sep="")
write.table(dfTraits, filename, sep="\t", row.names=TRUE, col.names=TRUE)

################################################################################
# Read the input data files.
################################################################################

dfRoots1 = read.xlsx2(processedDataFile1, sheetIndex=1, header=TRUE,
    stringsAsFactors=FALSE, colClasses=processedDataColClasses)

dfRoots2 = read.xlsx2(processedDataFile2, sheetIndex=1, header=TRUE,
    stringsAsFactors=FALSE, colClasses=processedDataColClasses)

################################################################################
# Get genotypes and experiments.
################################################################################

# Get genotypes and number of genotypes.
genotypes = unique(dfRoots1$genotype)
ngenotypes = length(genotypes) #how long is the vector (140 #genotypes)
printLog("There are", ngenotypes, "genotypes in total:\n")
printLog(genotypes, "\n", fill=70, labels="   ")

# Get experiment names and number of experiments.
experiments = unique(dfRoots1$experiment)
nexperiments = length(experiments)
printLog("There are", nexperiments, "experiments in total: ", paste(experiments, collapse=","), "\n")

################################################################################
# The data was obtained in multiple growth experiments.  Samples from different
# experiments were divvied up into four biological replicates numbered REP2
# through REP5 (REP1 was a practice rep).  Each biorep might have samples from
# more than one growth experiment.  The last one centimeter of most roots was
# harvested and RNA extracted for an RNAseq experiment.  Just prior to harvest,
# the plate was imaged.  Each root that grew, didn't hit plate walls, and wasn't
# obscured by labels was measured by tracing it and recording the centerline
# coordinates.  These coordinates were analyzed to generate the traits given in
# dfTraits.  There are no NA entries in the trait data, all roots in dfRoots
# have a value for each trait.
# In some cases, some ILs may have insufficient roots to be analyzed.  While the
# insufficient data could be passed to the modelling code, we prefer to remove
# those ILs here and list them up front so we know they have been excluded.
# Later, analyses using subsets of data may do the same thing, so we define a
# function here to subset the data.
################################################################################

# Make reduced data set from the data frame df that contains only those ILs for
# which there is at least minPtsPerGenotypePerExperiment roots in each experiment,
# and at least one experiment with minPtsPerGenotypePerExperiment roots.  Also,
# REMOVE ROOTS WHOSE "along" COLUMN IS TRUE (ROOT GREW ALONG ANOTHER).  Any
# data points in an experiment with fewer than "minPtsPerGenotypePerExperiment"
# data points are discarded.  Any IL with no data points remaining is gone.
# Return NULL if all data is removed, else return a list with these members:
#       data: data frame containing the remaining data, with several columns
#           converted to factors.  Certain columns are removed because we don't
#           need them here and they are in the way: filename, fileDate, comment,
#           plateRect, pts.
#       numRoots: matrix with genotypes (ILs) as row names, experiments as column
#           names, and entries the total number of roots for that IL in that
#           experiment AFTER removing cases with inadequate data, with all ILs
#           and experiments present in the matrix.
#       genotypes: the genotypes (ILs) remaining.
#       ngenotypes: number of ILs remaining.
#       experiments: the experiments remaining.
#       nexperiments: the number of experiments remaining.
reduceData = function(df, verbose=TRUE)
    {
    # Remove roots with along=TRUE.
    df = df[!df$along,]

    # Remove unneeded columns.
    df = df[, setdiff(colnames(df), c("filename", "fileDate", "comment", "plateRect", "pts"))]

    # Make list for returning data.
    rdata = list()

    # Save original genotypes and experiments.
	old.genotypes = unique(df$genotype)
	old.ngenotypes = length(old.genotypes)
	old.experiments = unique(df$experiment)
	old.nexperiments = length(old.experiments)

    # Count number of roots of each genotype in each experiment.
	numRoots = tapply(1:nrow(df), list(df$genotype, df$experiment), length)
	numRoots[is.na(numRoots)] = 0
    # Remove rows not meeting the minPtsPerGenotypePerExperiment requirement.
	df = df[apply(df, 1, function(d) numRoots[d["genotype"],d["experiment"]] >= minPtsPerGenotypePerExperiment),]
	# Check for no data left.
	if (nrow(df) == 0)
		{
		if (verbose)
    		printLog("No samples that satisfy the minimal data criteria!!!\n")
		return(NULL)
		}

	# Convert columns likely to be used in linear modelling to factors.
	df$replicate = factor(df$replicate)
	df$genotype = factor(df$genotype)
	df$platePos = factor(df$platePos)
	df$plateLetter = factor(df$plateLetter)
	df$germDay = factor(as.character(df$germDay))
	df$collide = factor(df$collide)
	df$experiment = factor(df$experiment)

	# Set m82 as reference level.
	if (!any(df$genotype == "M82"))
		{
		if (verbose)
    		printLog("No M82 samples that satisfy the minimal data criteria!!!\n")
		return(NULL)
		}
	df$genotype = relevel(df$genotype, ref="M82")

	# Save the reduced data set in rdata list.
	rdata$data = df

    # If verbose, display number of roots over all genotypes.
    if (verbose)
        {
        numRootsInGenotype = tapply(1:nrow(df), df$genotype, length)
        numRootsInGenotype = numRootsInGenotype[!(names(numRootsInGenotype) %in% parents)]
        printLog("   Minimum number of roots for any IL:", min(numRootsInGenotype), "\n")
        printLog("   Maximum number of roots for any IL:", max(numRootsInGenotype), "\n")
        printLog("   Mean number of roots for all ILs:", mean(numRootsInGenotype), "\n")
        printLog("   SD for number of roots for all ILs:", sd(numRootsInGenotype), "\n")
        printLog("\n")
        }

	# Count number of roots remaining for each genotype and experiment.
	rdata$numRoots = tapply(1:nrow(df), list(df$genotype, df$experiment), length)
    rdata$numRoots[is.na(rdata$numRoots)] = 0

	# If verbose, display number of roots over all experiments.
	if (verbose)
	    {
        numRoots = rdata$numRoots
        numRoots = numRoots[!rownames(numRoots) %in% parents,]
        printLog("   Min and max number of roots for ILs in each experiment:\n")
        for (experiment in colnames(numRoots))
            {
            x = numRoots[, experiment]
            x = x[x != 0]
            printLog("    ", format(experiment, width=6), " min:", format(min(x), width=3), " max:", format(max(x), width=3), "\n")
            }
        printLog("\n")
        }

    # Get remaining genotypes and experiments and save in rdata.
	new.genotypes = levels(df$genotype)
	new.ngenotypes = length(new.genotypes)
	new.experiments = levels(df$experiment)
	new.nexperiments = length(new.experiments)

	# Tell the user which genotypes were eliminated and which retained as a result of above.
	rmvd.genotypes = setdiff(old.genotypes, new.genotypes)
	rmvd.ngenotypes = length(rmvd.genotypes)
	printLog("   There are", new.ngenotypes, "genotypes remaining after removing those with insufficient data:\n")
	printLog(new.genotypes, "\n", fill=70, labels="   ")
	printLog("   There were", rmvd.ngenotypes, "genotypes removed.  They were:\n")
	printLog(rmvd.genotypes, "\n", fill=70, labels="   ")
	return(rdata)
	}

# Combine Siobhan's and Livingstone's datasets.
dfRoots = rbind(dfRoots1, dfRoots2)

# Now reduce the overall data set, removing ILs with insufficient data.
rdata = reduceData(dfRoots)
commonData = rdata$data

# Find the plates in common between Siobhan's and Livingstone's datasets, and
# remove data for all other plates.
commonPlates = intersect(unique(dfRoots1$plateID), unique(dfRoots2$plateID))
commonData = commonData[commonData$plateID %in% commonPlates,]
dim(commonData)

# Check if remaining data is PENN only.
gts = as.character(unique(commonData$genotype))
if (length(gts) != 1 || gts != "PENN")
    stop("Expected PENN only")

################################################################################
# Model testing.
################################################################################

# Is measurer significant?
printLog("Model testing: measurer effects PENN\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "measurer", sep=" ~ "))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = commonData
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    lm.1.s = summary(lm.1)
    lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    printLog(getPrint(lm.1.s$coefficients, "     "), "\n")
    }
# Not significant at all.  Try it with colliding roots removed.
printLog("Model testing: measurer effects PENN when colliding roots removed\n")
for (trait in traits)
    {
    printLog("**************************************************************", trait, "\n")
    formula = as.formula(paste(trait, "measurer", sep=" ~ "))
    printLog(" formula: ", Reduce(paste, deparse(formula)), "\n")
    data.r = commonData
    printLog(" Removing", sum(data.r$collide == TRUE), "colliding roots,",
        "leaving", sum(data.r$collide == FALSE), "roots\n")
    data.r = data.r[data.r$collide == "FALSE",]
    lm.1 = lm(formula, data=data.r)
  	printLog(getPrint(anova(lm.1), "    "), "\n")
    lm.1.s = summary(lm.1)
    lm.1.s$coefficients = as.data.frame(lm.1.s$coefficients)
    lm.1.s$coefficients$stars = getStars(lm.1.s$coefficients[, "Pr(>|t|)"])
    printLog(getPrint(lm.1.s$coefficients, "     "), "\n")
    }
# Not significant at all.
