################################################################################
# This file defines plate layout constants, the same values as used in the Root
# ImageJ macros file.
#
# This code is written in a way that makes them easy to tailor it for other
# projects.  This entire file can be copied and fairly simple changes made so
# that the file will serve for your project.
#
# Places to make changes are marked with XXXXX
################################################################################

################################################################################
# Set constants as per plate layout, same values as used in Root ImageJ macros
# file.
################################################################################

# Plate size (plates are square).
plateSize.in = 5  # XXXXX
plateSize.cm = plateSize.in * cmPerInch

# Set this TRUE only if the left and right halves of each plate are treated
# differently (e.g. different genotypes).  XXXXX
sided = FALSE

# Set this TRUE only if the plate contains two types of roots that are interleaved.
# There are four different types of plate layouts depending on the setting of
# "sided" and "interleave".  These for layouts are shown here by listing the seed
# number and seed type for each seed on the plate for each of the 4 layouts:
#       LAYOUT #1                   LAYOUT #2               LAYOUT #3                     LAYOUT #4
#  sided=F & interleave=F    sided=T & interleave=F   sided=F & interleave=T     sided=T & interleave=T
#  ----------------------    ----------------------   ------------------------   --------------------------------
#  All types = TDNA           TDNA       COL0
#  1  2  3  4  5  6          1  2  3    1  2  3       TDNA  1  2  3  4  5  6     TDNA  1  2  3    1  2  3  COL0
#  7  8  9  10 11 12         4  5  6    4  5  6       COL0  1  2  3  4  5  6     COL0  4  5  6    4  5  6  TDNA
#  13 14 15 16 17 18         7  8  9    7  8  9       TDNA  7  8  9  10 11 12    TDNA  7  8  9    7  8  9  COL0
#  19 20 21 22 23 24         10 11 12   10 11 12      COL0  7  8  9  10 11 12    COL0  10 11 12   10 11 12 TDNA
# XXXXX
interleave = FALSE

# The possible seed types.  If seed types are not used, set this to NA, else list
# the seed types.  XXXXX
seedTypes = NA

# Position of first seed from upper left corner of plate, and spacing of seeds
# in centimeters.  XXXXX
seed1X_L.cm = 2.04      # x-position of first seed on left side.
seed1X_R.cm = NA        # Use NA if sided is FALSE, else x-position of first seed on right side.
seed1Y.cm = 2.86        # y-position of first row of seeds.
seedSpaceX.cm = 1.42    # x-spacing of seeds.
seedSpaceY.cm = 0.1     # y-spacing of seeds.
seedRows = 1            # Number of rows of seeds on a plate.
seedCols = 7            # Number of columns of seeds on a plate.

# Number of seeds on a plate.
numSeedsPerPlate = seedRows*seedCols

# Number of columns on one side of the plate, only relevant if sided=TRUE.
colsOneSide = seedCols/2

################################################################################
# End of file.
################################################################################
cat("Include_plateLayout.R included\n")
